#include "joylink_stdio.h"
#include "joylink_ret_code.h"
#include "joylink_stdint.h"
#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_log.h"
#include "joylink_thread.h"

#include "joylink_utils.h"

#include "joylink_ble_sdk.h"
#include "joylink_ble.h"
#include "joylink_list.h"
#include "joylink_auth_crc.h"
#include "joylink3_auth_uECC.h"
#include "joylink_auth_uECC.h"
#include "joylink_aes_256.h"
#include "joylink_property_id.h"
#include "joylink_data_struct.h"

jl_packet_ctrl_t  g_comm_packet = {
    .packet_list = {NULL, NULL},
    .handle_list = {NULL, NULL},
    .recv_buffer = NULL,
};
jl_datum_t        g_dev_info = {.sdk_init_flag = 0};
jl_net_config_data_t  g_net_config_data;
static int32_t jl_get_packet(jl_packet_handle_t **handle, uint8_t operate);
static int32_t jl_packet_add_data(jl_packet_handle_t *handle, uint16_t tag, uint8_t length, uint8_t *value);
static int32_t jl_send_packet(jl_packet_handle_t *handle);

/**
 * @name:jl_dump_packet
 *
 * @param: packet_name
 * @param: packet
 */
static void jl_dump_packet(char *packet_name, uint8_t *packet_buffer)
{
    jl_packet_t packet;
    packet.seq = packet_buffer[0];
    packet.operation = packet_buffer[1];
    packet.len = packet_buffer[2] + (packet_buffer[3]<<8);
    packet.content = &(packet_buffer[4]);
    uint16_t crc = packet_buffer[JL_PACKET_HDR_LEN + packet.len] + (packet_buffer[JL_PACKET_HDR_LEN + packet.len + 1]<<8);
    jl_platform_printf("[%s]seq:%d, operation:0x%x, len:%d, crc:0x%02x\n", packet_name, packet.seq, packet.operation, packet.len, crc);

    int32_t offset = 0;
    jl_property_t property;
    while (offset < packet.len){
        property.tag = packet.content[offset] + (packet.content[offset+1]<<8);
        property.len = packet.content[offset+2];
        property.value = &(packet.content[offset+3]);
        jl_platform_printf("tag:0x%02x, len:%d, value:\n", property.tag, property.len);
        joylink_util_fmt_p("", property.value, property.len);
        offset = offset + property.len + 3;
    }
}


/**
 * @name:jl_dev_info_init
 *
 * @returns:
 */
static int32_t jl_dev_info_init(jl_dev_info_t *dev_info)
{
    jl_platform_memset(&g_dev_info,0x0,sizeof(g_dev_info));

    log_info("JL_SECURITY_LEVEL:%d", JL_SECURITY_LEVEL);

#if (JL_SECURITY_LEVEL == 1)
    jl_platform_memcpy(g_dev_info.shared_key, dev_info->shared_key, JL_SHARED_KEY_LEN);
    log_info("shared_key:%s", g_dev_info.shared_key);
#endif

#if (JL_SECURITY_LEVEL == JL_SECURITY_LEVEL_2)
    joylink_util_fmt_p("ecdh_key", g_dev_info.ecdh_key, sizeof(g_dev_info.ecdh_key));
#endif

    jl_platform_memcpy(g_dev_info.product_uuid, dev_info->product_uuid, JL_PRODUCT_UUID_LEN);
    jl_platform_memcpy(g_dev_info.mac, dev_info->mac, sizeof(g_dev_info.mac));
    log_info("product_uuid:%s", g_dev_info.product_uuid);
    jl_platform_printf("mac:%02X:%02X:%02X:%02X:%02X:%02X\r\n", g_dev_info.mac[0], g_dev_info.mac[1], \
                                                 g_dev_info.mac[2], g_dev_info.mac[3], \
                                                 g_dev_info.mac[4], g_dev_info.mac[5]);

    return E_RET_OK;
}

/**
 * @name:jl_get_manufacture_data
 *
 * @returns:
 */
static int32_t jl_get_manufacture_data(uint8_t *manufacture_data, uint32_t manufacture_data_len)
{

    if(JL_MANUFACTURE_DATA_LEN != manufacture_data_len){
        return E_RET_ERROR;
    }

    uint8_t chipinfo_1;
    uint8_t chipinfo_2;

    uint8_t offset = 0;
    uint8_t loop = 0;

    for (loop = 0; loop < JL_PRODUCT_UUID_LEN; loop++){
       manufacture_data[loop] = g_dev_info.product_uuid[JL_PRODUCT_UUID_LEN - 1 - loop];
    }
    offset += JL_PRODUCT_UUID_LEN;
    log_info("JL_PRODUCT_UUID:%s", g_dev_info.product_uuid);

    for (loop = 0; loop < JL_MAC_LEN; loop++){
        manufacture_data[loop + offset] = g_dev_info.mac[JL_MAC_LEN - 1 - loop];
    }
    offset += JL_MAC_LEN;

    uint8_t security_level = g_dev_info.security_level & 0x03;
    uint8_t protocol_version = (JL_PROTOCOL_VERSION & 0x0F)<<4;
    chipinfo_1 = security_level|protocol_version;
    log_info("security_level:%d, protocol_version:%d", security_level, protocol_version);
    manufacture_data[offset] = chipinfo_1;
    offset++;

    uint8_t work_mode = (JL_WORK_MODE & 0x0F)<<4;
    uint8_t wifi_connect = g_dev_info.wifi_connect & 0x0F;
    chipinfo_2 = work_mode|wifi_connect;
    log_info("work_mode:%d, wifi_connect:%d", work_mode, wifi_connect);
    manufacture_data[offset] = chipinfo_2;
    offset++;

    joylink_util_fmt_p("manufacture_data", manufacture_data, manufacture_data_len);

    if(manufacture_data_len != offset){
        return E_RET_ERROR;
    }

    return E_RET_OK;
}

int32_t jl_get_gap_config_data(jl_gap_data_t *gap_data)
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return E_RET_ERROR;
    }

    uint8_t jd_service_uuid16[2] = JL_GAP_SERVICE_UUID16;
    jl_platform_memcpy(gap_data->service_uuid16, jd_service_uuid16, 2);
    jl_get_manufacture_data(gap_data->manufacture_data, JL_MANUFACTURE_DATA_LEN);
    return E_RET_OK;
}

int32_t jl_get_gatt_config_data(jl_gatt_data_t *gatt_data)
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return E_RET_ERROR;
    }

    uint8_t jd_service_uuid128[16] = JL_GATT_SERVICE_UUID128;
    uint8_t jd_chra_uuid128_write[16] = JL_GATT_CHRA_UUID128_WRITE;
    uint8_t jd_chra_uuid128_indicate[16] = JL_GATT_CHRA_UUID128_INDICATE;
    jl_platform_memcpy(gatt_data->service_uuid128, jd_service_uuid128, 16);
    jl_platform_memcpy(gatt_data->chra_uuid128_write, jd_chra_uuid128_write, 16);
    jl_platform_memcpy(gatt_data->chra_uuid128_indicate, jd_chra_uuid128_indicate, 16);
    return E_RET_OK;
}

/**
 * @name:jl_packet_ctrl_init
 */
static void jl_packet_ctrl_init(void)
{
    INIT_LIST_HEAD(&g_comm_packet.packet_list);
    INIT_LIST_HEAD(&g_comm_packet.handle_list);

#ifdef JL_SEND_DATA_BY_THREAD
    g_comm_packet.send_mutex = jl_platform_mutex_create(0);
    g_comm_packet.send_sem = jl_platform_semaphore_create();
#endif

    g_comm_packet.recv_buffer = NULL;
    g_comm_packet.recv_last_frame = 0;
}

/**
 * @name:jl_secure_init
 *
 * @returns:
 */
static int32_t jl_secure_init(void)
{

#if (JL_SECURITY_LEVEL == 0)
    g_dev_info.security_level = JL_SECURITY_LEVEL_0;
#elif(JL_SECURITY_LEVEL == 1)
    g_dev_info.security_level = JL_SECURITY_LEVEL_1;
    key_expansion((uint8_t *)g_dev_info.shared_key, g_dev_info.share_expressed_key);
#elif (JL_SECURITY_LEVEL == JL_SECURITY_LEVEL_2)
    g_dev_info.security_level = JL_SECURITY_LEVEL_2;
    key_expansion(g_dev_info.ecdh_key, g_dev_info.share_expressed_key);

    jl3_uECC_set_rng((uECC_RNG_Function)joylink_util_random_buf);

    uECC_Curve curve = jl2_uECC_secp256r1();
    int32_t ret = jl3_uECC_make_key(g_dev_info.dev_pub_key, g_dev_info.dev_priv_key, curve);
    if (ret != 1){
        log_error("ecc make key error");
        return E_RET_ERROR;
    }

    joylink_util_fmt_p("dev_pub_key",g_dev_info.dev_pub_key,JL_ECC_PUBKEY_LEN);
    joylink_util_fmt_p("dev_priv_key",g_dev_info.dev_priv_key,JL_ECC_PRIKEY_LEN);
#endif
    return E_RET_OK;
}

/**
 * @name:jl_reset_receive_buffer
 *
 * @returns:
 */
static int32_t jl_reset_receive_buffer(void)
{
    log_info("reset receive buffer");
    if(NULL != g_comm_packet.recv_buffer){
        jl_platform_free(g_comm_packet.recv_buffer);
        g_comm_packet.recv_buffer = NULL;
    }
    g_comm_packet.recv_frame_count = 0;
    g_comm_packet.recv_last_frame = 0;
    return E_RET_OK;
}

/**
 * @name:jl_process_error_state
 *
 * @param: recv_flag
 *
 * @returns:
 */
int32_t jl_process_error_state(int32_t recv_flag)
{
    uint16_t tag = 0;//g_comm_packet.recv_buffer[4] + (g_comm_packet.recv_buffer[5]<<8);
    uint8_t ack_value = recv_flag + 200;

    jl_packet_handle_t *handle;
    jl_get_packet(&handle, JL_OPERATE_DEVICE_ERROR_RESPON);
    jl_packet_add_data(handle, tag, 1, &ack_value);
    jl_send_packet(handle);
    jl_reset_receive_buffer();
    return E_RET_OK;
}

/**
 * @name:jl_receive_frame
 *
 * @param: frame
 *
 * @returns:
 */
int32_t jl_process_frame(int32_t frame_len, uint8_t* frame)
{
    joylink_util_fmt_p("receive frame", frame, frame_len);

    if(JL_FRAME_LEN != frame_len){
        log_error("JL_ERR_FRAMELEN_ERROE frame_len:%d", frame_len);
        return JL_ERR_FRAMELEN_ERROE;
    }

    if (0 == frame[1] && frame[0] > 0){
        if(NULL != g_comm_packet.recv_buffer){
            jl_platform_free(g_comm_packet.recv_buffer);
        }
        g_comm_packet.recv_frame_count = frame[0];
        g_comm_packet.recv_last_frame = 0;
        uint16_t buffer_len = g_comm_packet.recv_frame_count*JL_FRAME_LEN;
        if (buffer_len > 0xFFFF){
              log_error("JL_ERR_OVER_BUFLEN buffer_len:%d", buffer_len);
              return JL_ERR_OVER_BUFLEN;
        }
        g_comm_packet.recv_buffer = jl_platform_malloc(buffer_len);
    }

    if(NULL == g_comm_packet.recv_buffer){
        log_error("JL_ERR_BUF_NULL");
        return JL_ERR_BUF_NULL;
    }


    if (frame[1] >= frame[0]){
        log_error("JL_ERR_NUM_OVERFLOW");
        return JL_ERR_NUM_OVERFLOW;
    }

    if ((g_comm_packet.recv_last_frame != frame[1]) ||
        (g_comm_packet.recv_last_frame >= g_comm_packet.recv_frame_count) ||
        (g_comm_packet.recv_frame_count != frame[0])){
        log_error("JL_ERR_NUM_UNORDER recv_last_frame: %d recv_frame_count:%d", g_comm_packet.recv_last_frame , g_comm_packet.recv_frame_count);
        return JL_ERR_NUM_UNORDER;
    }

    uint16_t receive_pointer = frame[1]*16;
    uint8_t *recv_buffer = g_comm_packet.recv_buffer;

    switch (frame[2]){
        case JL_KEY_TYPE_PLAIN:
            break;
#if (JL_SECURITY_LEVEL != JL_SECURITY_LEVEL_0)
        case JL_KEY_TYPE_PSK:
        case JL_KEY_TYPE_PSK_MIXER:
        case JL_KEY_TYPE_ECDH:{
                inv_cipher((uint8_t *)(frame+4), (uint8_t *)g_dev_info.share_expressed_key);
                joylink_util_fmt_p("receive inv_cipher frame", frame, JL_FRAME_LEN);
            }
            break;
#endif
        default:{
                log_error("JL_ERR_UNSUPPORT_KEY");
                return JL_ERR_UNSUPPORT_KEY;
            }
    }

    jl_platform_memcpy(recv_buffer+receive_pointer, frame+4, 16);

    g_comm_packet.recv_last_frame++;

    if (frame[1]+1 == frame[0]){
        return JL_RECEIVE_END;
    }else{
        return JL_RECEIVE_WAIT;
    }

}

/**
 * @name:jl_send_frame
 *
 * @param: packet
 *
 * @returns:
 */
int32_t jl_send_frame(jl_packet_node_t *packet)
{
    uint16_t i;
    uint8_t  frame[JL_FRAME_LEN];

    frame[0] = packet->frame_count;
    frame[1] = packet->last_frame;
    frame[2] = packet->key_type;
    frame[3] = 0;

    for (i = 0; i<16; i++){
        frame[4+i] = packet->buffer[i + packet->last_frame*16];
    }

    joylink_util_fmt_p("send frame", frame, JL_FRAME_LEN);
    switch (frame[2]){
        case JL_KEY_TYPE_PLAIN:
            break;
#if (JL_SECURITY_LEVEL != JL_SECURITY_LEVEL_0)
        case JL_KEY_TYPE_PSK:
        case JL_KEY_TYPE_PSK_MIXER:
        case JL_KEY_TYPE_ECDH:
            cipher((uint8_t *)(frame+4), (uint8_t *)g_dev_info.share_expressed_key);
            joylink_util_fmt_p("send cipher frame", frame, JL_FRAME_LEN);
            break;
#endif
    }

    if(0 == jl_ble_send_data(frame, 20)){
        packet->last_frame++;
    }
    return E_RET_OK;
}

#if (JL_SECURITY_LEVEL == JL_SECURITY_LEVEL_2)
/**
 * @name:jl_set_ecdh_key
 *
 * @param: appkey_compress
 * @param: len
 *
 * @returns:
 */
static int32_t jl_set_ecdh_key(uint8_t *appkey_compress,uint8_t len)  //to generate shared key
{
    int32_t ret = E_RET_OK;
    uint8_t *sharekey = NULL,*app_pub_key = NULL;
    uECC_Curve curve;

    if (len != JL_ECC_PUBKEY_PRESS_LEN){
        log_error("commpress publickey length error");
        ret = E_RET_ERROR;
        goto RET;
    }

    sharekey = (uint8_t *)jl_platform_malloc(JL_ECC_SHAREKEY_LEN*sizeof(uint8_t));
    if (sharekey == NULL){
        log_error("malloc error");
        ret = E_RET_ERROR;
        goto RET;
    }

    app_pub_key = (uint8_t *)jl_platform_malloc(JL_ECC_PUBKEY_LEN * sizeof(uint8_t));
    if (app_pub_key == NULL){
        log_error("malloc error");
        ret = E_RET_ERROR;
        goto RET;
    }

    curve = jl2_uECC_secp256r1();

    jl3_uECC_decompress(appkey_compress,app_pub_key,curve);
    jl3_uECC_shared_secret(app_pub_key, g_dev_info.dev_priv_key,sharekey,curve);

    jl_platform_memcpy(g_dev_info.ecdh_key, sharekey, JL_ECDH_KEY_LEN);

    key_expansion(g_dev_info.ecdh_key, g_dev_info.share_expressed_key);

    joylink_util_fmt_p("generate ecdh_key", g_dev_info.ecdh_key, sizeof(g_dev_info.ecdh_key));

RET:
    if (sharekey != NULL){
        jl_platform_free(sharekey);
        sharekey = NULL;
    }

    if (app_pub_key != NULL){
        jl_platform_free(app_pub_key);
        app_pub_key = NULL;
    }
    return ret;

}
#endif

/**
 * @name:jl_pubkey_compress
 *
 * @param: pubkey
 * @param: commpresskey
 *
 * @returns:
 */
#if (JL_SECURITY_LEVEL == JL_SECURITY_LEVEL_2)
static int32_t jl_pubkey_compress(uint8_t *pubkey,uint8_t *commpresskey)
{
    uECC_Curve curve = jl2_uECC_secp256r1();
    jl3_uECC_compress(pubkey, commpresskey, curve);
    return E_RET_OK;
}
#endif


#define JL_MULTIPLE  1000
int32_t jl_get_buf_timestamp(uint8_t *buf, uint8_t buf_len, jl_timestamp_t* timestamp)
{
    uint32_t sec = 0;
    uint32_t msec = 0;
    uint32_t tmp_sec = 0;
    uint32_t tmp_msec = 1;
    uint32_t integer_sec = 0;
    uint32_t value = 1;
    uint32_t i = 0;
    uint32_t j = 0;

    msec = buf[0];
    for(i=1; i<buf_len; i++){
        if(0 == buf[i]){
            continue;
        }

        for(j=0; j<i; j++){

           if(tmp_sec > 0){
               tmp_sec  = tmp_sec*0x100;
               tmp_msec = tmp_msec*0x100;
           }else{
               value = value*0x100;
               tmp_msec = value;
           }

           integer_sec = tmp_msec/JL_MULTIPLE;
           if(integer_sec > 0){
               tmp_sec += integer_sec;
               tmp_msec -= integer_sec*JL_MULTIPLE;
           }
        }

        tmp_sec = tmp_sec*buf[i];
        tmp_msec = tmp_msec*buf[i];
        integer_sec = tmp_msec/JL_MULTIPLE;
        if(integer_sec > 0){
            tmp_sec += integer_sec;
            tmp_msec -= integer_sec*JL_MULTIPLE;
        }

        sec += tmp_sec;
        msec += tmp_msec;
        integer_sec = msec/JL_MULTIPLE;
        if(integer_sec > 0){
            sec += integer_sec;
            msec -= integer_sec*JL_MULTIPLE;
        }
        tmp_sec = 0;
        tmp_msec = 0;
        value = 1;
    }

    timestamp->second = sec;
    timestamp->usecond = msec*1000;
    return 0;
}

void jl_process_config_data()
{
    if(g_dev_info.get_data_flag == (JL_GET_SSID_FLAG|JL_GET_PASSWORD_FLAG)){
        log_info("set config data\n");
        jl_ble_set_config_data(g_net_config_data.ssid, g_net_config_data.password);
        jl_platform_free(g_net_config_data.ssid);
        jl_platform_free(g_net_config_data.password);
        g_dev_info.get_data_flag = 0;
    }else{
        log_info("get_data_flag:%d\n", g_dev_info.get_data_flag);
    }
}

/**
 * @name:jl_process_property
 *
 * @param: property
 * @param: operate
 * @param: seq
 *
 * @returns:
 */

int32_t jl_process_property(void *resp_packet, uint32_t operate, uint16_t tag, uint8_t length, uint8_t *value)
{
    uint8_t status = E_RET_OK;

    switch (tag){
        case JL_PROPERTY_TAG_SSID:
        {
            g_net_config_data.ssid_len = length;
            g_net_config_data.ssid = (uint8_t *)jl_platform_malloc(length + 1);
            jl_platform_memset(g_net_config_data.ssid, 0, length + 1);
            joylink_util_reverse_buf(value, (uint8_t *)g_net_config_data.ssid, length);
            g_dev_info.get_data_flag |= JL_GET_SSID_FLAG;
            log_info("ssid: %s\n", g_net_config_data.ssid);
            break;
        }
        case JL_PROPERTY_TAG_PASSWORD:
        {
            g_net_config_data.password_len = length;
            g_net_config_data.password = (uint8_t *)jl_platform_malloc(length + 1);
            jl_platform_memset(g_net_config_data.password, 0, length + 1);
            joylink_util_reverse_buf(value, (uint8_t *)g_net_config_data.password, length);
            g_dev_info.get_data_flag |= JL_GET_PASSWORD_FLAG;
            log_info("password: %s\n", g_net_config_data.password);
            break;
        }
#if (JL_SECURITY_LEVEL == JL_SECURITY_LEVEL_2)
        case JL_PROPERTY_TAG_APP_PUBKEY:{
            if (length != JL_ECC_PUBKEY_PRESS_LEN){
                log_error("public len error");
                break;
            }
            status = jl_set_ecdh_key(value, length);
            break;
        }
        case JL_PROPERTY_TAG_DEV_PUBKEY:{
            uint8_t *compresskey = NULL;
            compresskey = (uint8_t *)jl_platform_malloc(JL_ECC_PUBKEY_PRESS_LEN * sizeof(uint8_t));
            if (compresskey == NULL){
                log_error("malloc error");
                break;
            }

            jl_pubkey_compress(g_dev_info.dev_pub_key,compresskey);

            jl_packet_add_data(resp_packet, tag, \
                                                    JL_ECC_PUBKEY_PRESS_LEN, \
                                                    compresskey);
            jl_platform_free(compresskey);
            compresskey = NULL;
            break;
        }
#endif
        default:
            break;
    }

    if(JL_OPERATE_PHONE_WRITE == operate){
        jl_packet_add_data(resp_packet, tag, sizeof(status), &status);
    }

    return E_RET_OK;
}

/********************** --JOYLINK BLE SDK FOR DEV APP-- **********************/
/**
 * @name:jl_indication_confirm_cb
 *
 * @returns:
 */
int32_t jl_send_confirm(void)
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return E_RET_ERROR;
    }

    int32_t ret = 0;
#ifndef JL_SEND_DATA_BY_THREAD
    if (!list_empty(&g_comm_packet.packet_list)){
        jl_packet_node_t *packet = container_of(g_comm_packet.packet_list.next, jl_packet_node_t, node);
        if (packet->last_frame < packet->frame_count){
            ret = jl_send_frame(packet);
        }else{
            list_del(&packet->node);
            jl_platform_free(packet->buffer);
            packet->buffer = NULL;
            jl_platform_free(packet);
            packet = NULL;
            log_info("send packet finished\r\n");
            jl_send_confirm();
        }
    }
#endif
    return ret;
}

/**
 * @name:jl_get_packet
 *
 * @returns:
 */
static int32_t jl_get_packet(jl_packet_handle_t **handle, uint8_t operate)
{
    jl_packet_handle_t *packet_handle = (void *)jl_platform_malloc(sizeof(jl_packet_handle_t));

    if(NULL == packet_handle){
        *handle = 0;
        log_error("get_packet fail\n");
        return E_RET_ERROR;
    }
    packet_handle->operate = operate;
    packet_handle->property_len = 0;
    INIT_LIST_HEAD(&packet_handle->property_list);
    list_add_tail(&packet_handle->node, &g_comm_packet.handle_list);
    *handle = packet_handle;
    return E_RET_OK;
}

/**
 * @name:jl_packet_add_data
 *
 * @param: packet
 * @param: tag
 * @param: length
 * @param: value
 *
 * @returns:
 */
static int32_t jl_packet_add_data(jl_packet_handle_t *handle, uint16_t tag, uint8_t length, uint8_t *value)
{
    if (NULL == handle || 0 == tag || 0 == length || NULL == value ){
        log_error("parameters error handle:%p, tag:0x02%d, length:%d, value:%p", handle, tag, length, value);
        return E_RET_ERROR;
    }

    handle->property_len += (JL_PROPERTY_HDR_LEN + length);

    jl_property_node_t *property_node = (jl_property_node_t *)jl_platform_malloc(sizeof(jl_property_node_t));
    if (NULL == property_node){
        log_error("malloc property_node fail");
        return E_RET_ERROR;
    }

    property_node->buffer = (uint8_t *)jl_platform_malloc(JL_PROPERTY_HDR_LEN + length);
    if (NULL == property_node->buffer){
        jl_platform_free(property_node);
        log_error("malloc property_node->buffer fail");
        return E_RET_ERROR;
    }

    property_node->buffer[0] = tag&0xFF;
    property_node->buffer[1] = tag>>8;
    property_node->buffer[2] = length;
    jl_platform_memcpy(property_node->buffer + JL_PROPERTY_HDR_LEN, value, length);
    list_add_tail(&property_node->node, &handle->property_list);
    return E_RET_OK;
}

/**
 * @name:jl_send_packet
 *
 * @param: packet
 *
 * @returns:
 */
static int32_t jl_send_packet(jl_packet_handle_t *handle)
{
    if (NULL == handle){
        log_error("packet NULL");
        return E_RET_ERROR;
    }

    jl_packet_handle_t *packet_handle = NULL;
    struct list_head *list = g_comm_packet.handle_list.next;
    while (list != &g_comm_packet.handle_list){
        if(handle == container_of(list, jl_packet_handle_t, node)){
            packet_handle = handle;
            break;
        }
        list = list->next;
    }

    if(NULL == packet_handle){
        log_error("packet_handle NULL");
        return E_RET_ERROR;
    }

    jl_packet_node_t *packet_node = (jl_packet_node_t *)jl_platform_malloc(sizeof(jl_packet_node_t));

    if (NULL == packet_node){
        log_error("malloc error");
        goto RET;
    }
    jl_platform_memset(packet_node, 0, sizeof(jl_packet_node_t));

    uint16_t packet_length = JL_PACKET_HDR_LEN + packet_handle->property_len + JL_PACKET_CRC_LEN;
    if(packet_length > 0xFFFF){
        log_error("packet_length error");
        goto RET;
    }

    if (packet_length%16){
        packet_node->frame_count = packet_length/16 + 1;
    }else if(packet_length > 16){
        packet_node->frame_count = packet_length/16;
    }else{
        packet_node->frame_count = 1;
    }

    uint16_t buffer_length = 16*packet_node->frame_count;
    packet_node->buffer = (uint8_t*)jl_platform_malloc(buffer_length);
    if (packet_node->buffer == NULL){
        log_error("malloc error");
        goto RET;
    }
    jl_platform_memset(&(packet_node->buffer[packet_length]), 0x0, buffer_length - packet_length);

    packet_node->buffer[0] = g_dev_info.seq_up;
    packet_node->buffer[1] = packet_handle->operate;
    packet_node->buffer[2] = (packet_handle->property_len)&0xFF;
    packet_node->buffer[3] = (packet_handle->property_len)>>8;

    uint32_t offset = JL_PACKET_HDR_LEN;
    jl_property_node_t *property_node = NULL;
    uint32_t property_buf_len = 0;
    while (!list_empty(&packet_handle->property_list)){
        property_node = container_of(packet_handle->property_list.next, jl_property_node_t, node);
        property_buf_len = property_node->buffer[2] + JL_PROPERTY_HDR_LEN;
        jl_platform_memcpy(packet_node->buffer  + offset, property_node->buffer, property_buf_len);
        offset+=property_buf_len;

        list_del(&property_node->node);
        jl_platform_free(property_node->buffer);
        property_node->buffer = NULL;
        jl_platform_free(property_node);
        property_node = NULL;
    }

    uint16_t crc = CRC16((uint8_t*)(packet_node->buffer + JL_PACKET_HDR_LEN), packet_handle->property_len);
    packet_node->buffer[JL_PACKET_HDR_LEN + packet_handle->property_len] = crc&0xFF;
    packet_node->buffer[JL_PACKET_HDR_LEN + packet_handle->property_len + 1] = crc>>8;

    list_del(&packet_handle->node);
    jl_platform_free(packet_handle);

    uint16_t first_tag = packet_node->buffer[JL_PACKET_HDR_LEN] + (packet_node->buffer[JL_PACKET_HDR_LEN + 1]<<8);
    if ((first_tag == 0xfef9)||(first_tag == 0xfef8)){
        packet_node->key_type = JL_KEY_TYPE_PLAIN;
    }else if ((first_tag >= 0xa000) && (first_tag <= 0xffff)){
        packet_node->key_type = g_dev_info.security_level;
    }else{
        if (g_dev_info.security_level == JL_SECURITY_LEVEL_2){
            packet_node->key_type = JL_KEY_TYPE_ECDH_SESSION;
        }else if (g_dev_info.security_level == JL_SECURITY_LEVEL_1){
            packet_node->key_type = JL_KEY_TYPE_PSK_SESSION;
        }else{
            packet_node->key_type = g_dev_info.security_level;
        }
    }


    jl_dump_packet("send packet", packet_node->buffer);

#ifndef JL_SEND_DATA_BY_THREAD
    if (list_empty(&g_comm_packet.packet_list)){
        jl_send_frame(packet_node);
    }
    list_add_tail(&packet_node->node, &g_comm_packet.packet_list);
#else
    jl_platform_mutex_lock(g_comm_packet.send_mutex);
    list_add_tail(&packet_node->node, &g_comm_packet.packet_list);
    jl_platform_mutex_unlock(g_comm_packet.send_mutex);
    jl_platform_semaphore_post(g_comm_packet.send_sem);
#endif
    return E_RET_OK;

RET:
    if(NULL != packet_handle){
        list_del(&packet_handle->node);
        jl_platform_free(packet_handle);
    }

    if (packet_node != NULL){
        if(packet_node->buffer != NULL){
            jl_platform_free(packet_node->buffer);
        }
        jl_platform_free(packet_node);
    }

    return E_RET_ERROR;
}

static int32_t jl_process_packet()
{
    jl_dump_packet("receive packet", g_comm_packet.recv_buffer);
    uint8_t operate = g_comm_packet.recv_buffer[1];
    uint16_t len = g_comm_packet.recv_buffer[2] + (g_comm_packet.recv_buffer[3]<<8);
    uint8_t *content = &(g_comm_packet.recv_buffer[4]);
    int recv_packet_len = g_comm_packet.recv_last_frame*JL_FRAME_LEN;
    g_dev_info.get_data_flag = 0;
    if ((len > recv_packet_len - 4) || len > 0xFFFF - 4){
        log_error("recv_packet lenght error !!! max_len:%d, len:%d", recv_packet_len, len);
        jl_process_error_state(JL_ERR_OVER_BUFLEN);
        return E_RET_ERROR;
    }

    uint16_t crc_rcv = g_comm_packet.recv_buffer[JL_PACKET_HDR_LEN + len] + (g_comm_packet.recv_buffer[JL_PACKET_HDR_LEN + len + 1]<<8);
    uint16_t crc_src = CRC16(g_comm_packet.recv_buffer, JL_PACKET_HDR_LEN + len);
    if(crc_rcv != crc_src){//Android手机没有这个校验暂时不做判断，后面有的话就加上
        //log_error("CRC16 error !!! crc_rcv:0x%X, crc_src:0x%X", crc_rcv, crc_src);
        //jl_process_error_state(JL_ERR_CRC_CHECK);
        //return E_RET_ERROR;
    }

    log_info("operate:0x%x", operate);

    jl_packet_handle_t *packet_handle = NULL;
    if(JL_OPERATE_PHONE_READ == operate){
        jl_get_packet(&packet_handle, JL_OPERATE_DEVICE_READ_RESPON);
    }else if(JL_OPERATE_PHONE_WRITE == operate){
        jl_get_packet(&packet_handle, JL_OPERATE_DEVICE_WRITE_RESPON);
    }else{
        jl_reset_receive_buffer();
        return E_RET_OK;
    }

    if(NULL == packet_handle){
        log_error("packet_handle is NULL");
        jl_process_error_state(JL_ERR_BUF_NULL);
        return E_RET_ERROR;
    }

    int32_t offset = 0;
    uint16_t tag = 0;
    uint8_t length = 0;
    uint8_t *value = NULL;
    while (offset < len){
        tag = content[offset] + (content[offset+1]<<8);
        length = content[offset+2];
        value = content + offset + 3;
        jl_process_property(packet_handle, operate, tag, length, value);
        offset = offset + length + 3;
    }

    jl_reset_receive_buffer();

    if (packet_handle){
        jl_send_packet(packet_handle);
    }

    jl_process_config_data();

    return E_RET_OK;
}
/**
 * @name:jl_write_data
 *
 * @param:
 *
 * @returns:
 */
int32_t jl_write_data(uint8_t* data, int32_t data_len)
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return E_RET_ERROR;
    }

    int32_t recv_flag = jl_process_frame(data_len, data);
    if (JL_RECEIVE_END == recv_flag){
        jl_process_packet();
    } else if (JL_RECEIVE_WAIT != recv_flag){
        jl_process_error_state(recv_flag);
    }
    return E_RET_OK;
}

static int32_t jl_reset_data_list(void)
{
    log_info("reset packet_list:%p, next:%p", &g_comm_packet.packet_list, &g_comm_packet.packet_list.next);
    while (!list_empty(&g_comm_packet.packet_list)){
        jl_packet_node_t *packet = container_of(g_comm_packet.packet_list.next, jl_packet_node_t, node);
        list_del(&packet->node);
        jl_platform_free(packet->buffer);
        packet->buffer = NULL;
        jl_platform_free(packet);
        packet = NULL;
        log_info("delete packet_list");
    }
    INIT_LIST_HEAD(&g_comm_packet.packet_list);

    log_info("reset handle_list:%p, next:%p", &g_comm_packet.handle_list, &g_comm_packet.handle_list.next);
    while (!list_empty(&g_comm_packet.handle_list)){
        jl_packet_handle_t *packet_handle = container_of(g_comm_packet.handle_list.next, jl_packet_handle_t, node);
        while (!list_empty(&packet_handle->property_list)){
            jl_property_node_t *property_node = container_of(packet_handle->property_list.next, jl_property_node_t, node);
            list_del(&property_node->node);
            jl_platform_free(property_node);
            property_node = NULL;
        }
        list_del(&packet_handle->node);
        jl_platform_free(packet_handle);
        packet_handle = NULL;
        log_info("delete handle_list");
    }
    INIT_LIST_HEAD(&g_comm_packet.handle_list);
    return E_RET_OK;
}

int32_t jl_disconnet_reset()
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return E_RET_ERROR;
    }

    log_info("jl_disconnet_reset");

    jl_reset_data_list();
    jl_reset_receive_buffer();
    return E_RET_OK;
}

int32_t jl_send_net_config_state(E_JL_NET_CONF_ST state, uint8_t* data, int32_t data_len)
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return E_RET_ERROR;
    }

    int ret = 0;
    jl_packet_handle_t *handle = NULL;

    if(jl_get_packet(&handle, JL_OPERATE_DEVICE_SEND) != 0){
        log_info("send handle error\n");
        return -1;
    }

    switch(state)
    {
        case E_JL_NET_CONF_ST_TIMEOUT_EXIT:
        case E_JL_NET_CONF_ST_EXIT:
        case E_JL_NET_CONF_ST_WIFI_CONNECT_TIMEOUT :
        case E_JL_NET_CONF_ST_WIFI_CONNECT_SSID_ERROR :
        case E_JL_NET_CONF_ST_WIFI_CONNECT_FAILED :
        case E_JL_NET_CONF_ST_WIFI_CONNECT_START  :
        case E_JL_NET_CONF_ST_WIFI_CONNECT_PSK_ERROR      :
        case E_JL_NET_CONF_ST_WIFI_CONNECT_WAN_ERROR      :
        case E_JL_NET_CONF_ST_WIFI_CONNECT_SUCCEED:
        {
            uint8_t value = state;
            ret |= jl_packet_add_data(handle, JL_PROPERTY_TAG_WIFI_STATUS, sizeof(value), &value);
            break;
        }
        default:
            break;
    }

    ret |= jl_send_packet(handle);

    return ret;
}

#ifdef JL_SEND_DATA_BY_THREAD
void * jl_send_thread(void *arg)
{
    if(!g_dev_info.sdk_init_flag){
        log_error("sdk isn't init");
        return NULL;
    }

    while(1){
        jl_platform_semaphore_wait(g_comm_packet.send_sem, 1000);

        jl_platform_mutex_lock(g_comm_packet.send_mutex);
        if (!list_empty(&g_comm_packet.packet_list)){
            jl_packet_node_t *packet = container_of(g_comm_packet.packet_list.next, jl_packet_node_t, node);
            if (packet->last_frame < packet->frame_count){
                jl_send_frame(packet);
            }else{
                list_del(&packet->node);
                jl_platform_free(packet->buffer);
                packet->buffer = NULL;
                jl_platform_free(packet);
                packet = NULL;
                log_info("send packet finished\r\n");
            }
            jl_platform_semaphore_post(g_comm_packet.send_sem);
        }
        jl_platform_mutex_unlock(g_comm_packet.send_mutex);
    }
    return NULL;
}
#endif

int32_t jl_init(jl_dev_info_t *dev_info)
{
    log_info("-------------  joylink bt version %s  -------------\n",JL_BLE_SDK_VERSION);
    jl_dev_info_init(dev_info);
    jl_packet_ctrl_init();
    jl_secure_init();
    g_dev_info.sdk_init_flag = 1;
    log_info("jl init done");
    return E_RET_OK;
}

int32_t jl_deinit()
{
    jl_reset_data_list();
    jl_reset_receive_buffer();

#ifdef JL_SEND_DATA_BY_THREAD
    jl_platform_mutex_delete(g_comm_packet.send_mutex);
    jl_platform_semaphore_destroy(g_comm_packet.send_sem);
#endif

    jl_platform_memset(&g_dev_info, 0x0, sizeof(g_dev_info));
    log_info("jl deinit done");
    return E_RET_OK;
}

#ifdef __cplusplus
}
#endif

