/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: joylink 协议 设备连接地址分发相关实现
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#include <stdbool.h>
#include "joylink_protocol.h"
#include "joylink_proto_internal.h"

#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_stdint.h"
#include "joylink_log.h"
#include "joylink_flash.h"

#include "cJSON.h"

/**
 * @brief  获取hostname
 * @param[in/out] hostname 指针
 * @return
 *    返回值 hostname 指针
 * @note
 */
char *joylink_linkaddress_hostname_get(char *hostname)
{
    int32_t len = 0;

    len = jl_flash_GetString("curr_host", hostname);
    if (len) {
        log_info("hostname:%s, len:%d\n", hostname, len);
        return hostname;
    } else {
        log_info("curr_host is NULL, use default");
        return NULL;
    }
}

/**
 * @brief  设置hostname
 * @param[in] hostname 指针
 * @return success 0, failed -1
 * @note
 */
int32_t joylink_linkaddress_hostname_set(char *hostname)
{
    int32_t ret = JOYLINK_RET_FAILED;
    char curr_host[64] = {0};
    int32_t len = 0;

    //if curr NULL, set hostname in curr & last
    //if curr exist, set curr in last, set hostname in curr
    len = jl_flash_GetString("curr_host", curr_host);
    jl_flash_ClearKey("last_host");
    ret = jl_flash_SetString("last_host", len > 0 ? curr_host : hostname);
    if (ret == JOYLINK_RET_SUCCESS) {
        log_info("set last_host:%s\n", len > 0 ? curr_host : hostname);
    } else {
        log_error("set last_host error\n");
        goto RET;
    }

    ret = jl_flash_SetString("curr_host", hostname);
    if (ret == JOYLINK_RET_SUCCESS) {
        log_info("set curr_host:%s, len:%d\n", hostname, jl_platform_strlen(hostname));
    } else {
        log_error("set curr_host error\n");
    }

RET:
    return ret;
}

/**
 * @brief  获取port
 * @param[in/out] port 指针
 * @return
 *    返回值 success port, failed -1
 * @note
 */
int32_t joylink_linkaddress_port_get(int32_t *port)
{
    *port = jl_flash_GetInt32("curr_port", 0);
    if (*port > 0) {
        log_info("get port:%d\n", *port);
    } else {
        log_info("curr_port is NULL, use default");
    }

    return *port;
}

/**
 * @brief  设置port
 * @param[in] port
 * @return success 0, failed -1
 * @note
 */
int32_t joylink_linkaddress_port_set(int32_t port)
{
    int32_t curr_port = 0;
    int32_t ret = JOYLINK_RET_FAILED;

    //if curr NULL, set port in curr & last
    //if curr exist, set curr in last, set port in curr
    curr_port = jl_flash_GetInt32("curr_port", 0);
    jl_flash_ClearKey("last_port");
    ret = jl_flash_SetInt32("last_port", curr_port > 0 ? curr_port : port);
    if (ret == JOYLINK_RET_SUCCESS) {
        log_info("set last_port:%d\n", curr_port > 0 ? curr_port : port);
    } else {
        log_error("set last_port error\n");
        goto RET;
    }

    ret = jl_flash_SetInt32("curr_port", port);
    if (ret == JOYLINK_RET_SUCCESS) {
        log_info("set curr_port:%d\n", port);
    } else {
        log_error("set curr_port error\n");
    }

RET:
    return ret;
}

/**
 * @brief  获取protocol
 * @param[in/out] protocol 指针
 * @return
 *    返回值 protocol 指针
 * @note
 */
char *joylink_linkaddress_protocol_get(char *protocol)
{
    int32_t len = 0;

    len = jl_flash_GetString("curr_prot", protocol);
    if (len) {
        log_info("protocol:%s, len:%d\n", protocol, len);
        return protocol;
    } else {
        log_error("curr_prot is NULL, use default");
        return NULL;
    }
}

/**
 * @brief  设置protocol
 * @param[in] protocol 指针
 * @return success 0, failed -1
 * @note
 */
int32_t joylink_linkaddress_protocol_set(char *protocol)
{
    int32_t ret = JOYLINK_RET_FAILED;
    char curr_prot[16] = {0};
    int32_t len = 0;

    //if curr NULL, set protocol in curr & last
    //if curr exist, set curr in last, set protocol in curr
    len = jl_flash_GetString("curr_prot", curr_prot);
    jl_flash_ClearKey("last_prot");
    ret = jl_flash_SetString("last_prot", len > 0 ? curr_prot : protocol);
    if (ret == JOYLINK_RET_SUCCESS) {
        log_info("set last_prot:%s\n", len > 0 ? curr_prot : protocol);
    } else {
        log_error("set last_prot error\n");
        goto RET;
    }

    ret = jl_flash_SetString("curr_prot", protocol);
    if (ret == JOYLINK_RET_SUCCESS) {
        log_info("set curr_prot:%s, len:%d\n", protocol, jl_platform_strlen(protocol));
    } else {
        log_error("set curr_prot error\n");
    }

RET:
    return ret;
}

/**
 * @brief  linkaddress curr last 互换
 * @param[in] protocol 指针
 * @return success 0, failed -1
 * @note
 */
int32_t joylink_linkaddress_exchange(void)
{
    int32_t ret = JOYLINK_RET_FAILED;
    char curr_tmp[64], last_tmp[64];
    int32_t curr_port = 0, last_port = 0;

    //exchange protocol
    jl_platform_memset(curr_tmp, 0, sizeof(curr_tmp));
    jl_platform_memset(last_tmp, 0, sizeof(last_tmp));
    jl_flash_GetString("curr_prot", curr_tmp);
    jl_flash_GetString("last_prot", last_tmp);
    jl_flash_ClearKey("last_prot");
    ret = jl_flash_SetString("last_prot", curr_tmp);
    if(ret) {
        log_error("set last_prot error\n");
        goto RET;
    } else {
        log_info("set last_prot:%s\n", curr_tmp);
    }

    if (jl_platform_strlen(last_tmp)) {
        jl_flash_ClearKey("curr_prot");
        ret = jl_flash_SetString("curr_prot", last_tmp);
        if(ret) {
            log_error("set curr_prot error\n");
            goto RET;
        } else {
            log_info("set curr_prot:%s\n", last_tmp);
        }       
    }

    //exchange hostname
    jl_platform_memset(curr_tmp, 0, sizeof(curr_tmp));
    jl_platform_memset(last_tmp, 0, sizeof(last_tmp));
    jl_flash_GetString("curr_host", curr_tmp);
    jl_flash_GetString("last_host", last_tmp);
    jl_flash_ClearKey("last_host");
    ret = jl_flash_SetString("last_host", curr_tmp);
    if(ret) {
        log_error("set last_host error\n");
        goto RET;
    } else {
        log_info("set last_host:%s\n", curr_tmp);
    }

    if (jl_platform_strlen(last_tmp)) {
        jl_flash_ClearKey("curr_host");
        ret = jl_flash_SetString("curr_host", last_tmp);
        if(ret) {
            log_error("set curr_host error\n");
            goto RET;
        } else {
            log_info("set curr_host:%s\n", last_tmp);
        }        
    }

    //exchange port
    curr_port = jl_flash_GetInt32("curr_port", 0);
    last_port = jl_flash_GetInt32("last_port", 0);
    jl_flash_ClearKey("last_port");
    ret = jl_flash_SetInt32("last_port", curr_port);
    if(ret) {
        log_error("set last_port error\n");
        goto RET;
    } else {
        log_info("set last_port:%d\n", curr_port);
    }   

    if (last_port > 0) {
        jl_flash_ClearKey("curr_port");
        ret = jl_flash_SetInt32("curr_port", last_port);
        if(ret) {
            log_error("set curr_port error\n");
            goto RET;
        } else {
            log_info("set curr_port:%d\n", last_port);
        }           
    }

RET:
    return ret;
}

/**
 * @brief   linkaddress/reissue消息主题 解析linkaddress/reissue消息
 *
 * @param[in] in_json: 输入的json串
 * @param[out] out_set: 用于存解析结果的 linkaddress/reissue结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note None.
 */
int32_t joylink_proto_parse_linkaddress_reissue(char *in_json, JoyProtoLinkReissue_t *out_set)
{
    int32_t ret = JOYLINK_RET_FAILED;
    if (NULL == in_json || NULL == out_set) {
        goto RET;
    }
    cJSON *payload = cJSON_Parse(in_json);
    if (NULL == payload) {
        goto RET;
    }

    cJSON *pV = NULL;

    // 解析 deviceId
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    if (out_set->deviceId == NULL) {
        out_set->deviceId = (char *) jl_platform_malloc((jl_platform_strlen(pV->valuestring) + 2) * sizeof(char));
    }
    jl_platform_strcpy(out_set->deviceId, pV->valuestring);

    // 解析 messageId
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    if (out_set->messageId == NULL) {
        out_set->messageId = (char *) jl_platform_malloc((jl_platform_strlen(pV->valuestring) + 2) * sizeof(char));
    }
    jl_platform_strcpy(out_set->messageId, pV->valuestring);

    // 解析 timestamp
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_TIMESTAMP);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    out_set->timestamp = pV->valueint;

    // 解析 linkAddress
    pV = cJSON_GetObjectItem(payload, "linkAddress");
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    out_set->linkAddress = (JoyProtoLinkAddress_t *) jl_platform_malloc(
                            sizeof(JoyProtoLinkAddress_t));
    jl_platform_memset(out_set->linkAddress, 0, sizeof(JoyProtoLinkAddress_t));
    cJSON *Protocol, *Host, *Port;
    // 解析 linkAddress protocol元素
    Protocol = cJSON_GetObjectItem(pV, "protocol");
    if (NULL == Protocol) {
        cJSON_Delete(payload);
        goto RET;
    }
    out_set->linkAddress->protocol = (char *) jl_platform_malloc(
        (jl_platform_strlen(Protocol->valuestring) + 1) * sizeof(char));
    jl_platform_strcpy(out_set->linkAddress->protocol, Protocol->valuestring);
    // 解析 linkAddress hostname元素
    Host = cJSON_GetObjectItem(pV, "hostname");
    if (NULL == Host) {
        cJSON_Delete(payload);
        goto RET;
    }
    out_set->linkAddress->hostname = (char *) jl_platform_malloc(
        (jl_platform_strlen(Host->valuestring) + 1) * sizeof(char));
    jl_platform_strcpy(out_set->linkAddress->hostname, Host->valuestring);
    // 解析 linkAddress port元素
    Port = cJSON_GetObjectItem(pV, "port");
    if (NULL == Port) {
        cJSON_Delete(payload);
        goto RET;
    }
    out_set->linkAddress->port = Port->valueint;

    // 删除 payload
    cJSON_Delete(payload);

    ret = JOYLINK_RET_SUCCESS;
RET:
    return ret;
}

/**
 * @brief    linkaddress/reissue消息主题 释放linkaddress/reissue结构体 成员变量的内存空间
 *
 * @param[in] in_set: linkaddress/reissue结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_linkaddress_reissue(JoyProtoLinkReissue_t *in_set)
{
    if (in_set != NULL) {
        if (in_set->deviceId != NULL) {
            jl_platform_free(in_set->deviceId);
            in_set->deviceId = NULL;
        }
        if (in_set->messageId != NULL) {
            jl_platform_free(in_set->messageId);
            in_set->messageId = NULL;
        }
        if (in_set->linkAddress != NULL) {
            if (in_set->linkAddress->hostname != NULL) {
                jl_platform_free(in_set->linkAddress->hostname);
                in_set->linkAddress->hostname = NULL;
            }
            if (in_set->linkAddress->protocol != NULL) {
                jl_platform_free(in_set->linkAddress->protocol);
                in_set->linkAddress->protocol = NULL;
            }
            jl_platform_free(in_set->linkAddress);
            in_set->linkAddress = NULL;
        }
    }
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   linkaddress(reissue/response)消息主题 释放reissue/response结构体 成员变量的内存空间
 *
 * @param[in] in_res: reissue/response结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_linkaddress_reissue_res(JoyProtoLinkReissueRes_t *in_res)
{
    if (in_res != NULL) {
        if (in_res->deviceId != NULL) {
            jl_platform_free(in_res->deviceId);
            in_res->deviceId = NULL;
        }
        if (in_res->messageId != NULL) {
            jl_platform_free(in_res->messageId);
            in_res->messageId = NULL;
        }
        if (in_res->message != NULL) {
            jl_platform_free(in_res->message);
            in_res->message = NULL;
        }
    }
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   linkaddress(reissue/response)消息主题 打包reissue/response响应
 *
 * @param[in] in_res: 待打包的 reissue/response响应 结构体指针
 * @return
 *    打包后的json串指针
 * @see None.
 * @note None.
 */
char   *joylink_proto_pack_linkaddress_reissue_res(JoyProtoLinkReissueRes_t *in_res)
{
    if(NULL == in_res) {
        return NULL;
    }

    cJSON *root;
    char *out  = NULL;
    root = cJSON_CreateObject();
    if(NULL == root){
        goto RET;
    }

    // 添加 deviceId
    if (in_res->deviceId != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID, in_res->deviceId);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID, "");
    }

    // 添加 messageId
    if (in_res->messageId != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID, in_res->messageId);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID, "");
    }

    // 添加 timestamp
    cJSON_AddNumberToObject(root, JOYLINK_PAYLOAD_NODE_NAME_TIMESTAMP, in_res->timestamp);

    // 添加 message (可选)
    if (in_res->message != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG, in_res->message);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG, "");
    }

    // 添加 code
    cJSON_AddNumberToObject(root, JOYLINK_PAYLOAD_NODE_NAME_CODE, in_res->code);

    // 转换成字符串
    out = cJSON_Print(root);

    // 删除 root
    cJSON_Delete(root);

RET:
    return out;
}

// end of file
