/*Copyright (c) 2015-2050, JD All rights reserved.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License. */

/* --------------------------------------------------
 * @brief: joylink 协议 设备NTP授时相关实现
 *
 * @version: 1.0
 *
 * --------------------------------------------------
 */

#include <stdbool.h>
#include "joylink_protocol.h"
#include "joylink_proto_internal.h"

#include "joylink_string.h"
#include "joylink_memory.h"
#include "joylink_stdint.h"
#include "joylink_log.h"

#include "cJSON.h"

/**
 * @brief   设备NTP授时 时间戳转成字符串
 *
 * @param[in] time_stamp: 时间戳指针
 * @param[in] time_str: 时间戳字符串指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
static int32_t joylink_ntp_stamp_to_string(jl_time_stamp_t *time_stamp, char *time_str)
{
    if (time_stamp == NULL || time_str == NULL) {
        return JOYLINK_RET_FAILED;    
    }

    jl_platform_sprintf(time_str, "%010d%03d", time_stamp->second, time_stamp->ms);

    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   设备NTP授时 字符串转成时间戳
 *
 * @param[in] time_str: 时间戳字符串指针
 * @param[in] time_stamp: 时间戳指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
static int32_t joylink_ntp_string_to_stamp(char *time_str, jl_time_stamp_t *time_stamp)
{
    if (time_stamp == NULL || time_str == NULL) {
        return JOYLINK_RET_FAILED;    
    }
    int32_t str_len = jl_platform_strlen(time_str);
    if (str_len <= 3) {
        log_error("The Length of Time String is too short!!");
        return JOYLINK_RET_FAILED;
    }
    char str_sec[16];
    jl_platform_memset(str_sec, 0, 16 * sizeof(char));
    jl_platform_memcpy(str_sec, time_str, (str_len-3) * sizeof(char));
    
    time_stamp->second = (uint32_t) jl_platform_atoi(str_sec);
    time_stamp->ms = (uint32_t) jl_platform_atoi(&time_str[str_len-3]);

    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   设备NTP授时 释放 请求信息结构体成员变量
 *
 * @param[in] in_req: NTP请求信息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_ntp_req(JoyProtoNTPReq *in_req)
{
    if (in_req != NULL) {
        if (in_req->deviceId != NULL) {
            jl_platform_free(in_req->deviceId);
            in_req->deviceId = NULL;
        }
        if (in_req->messageId != NULL) {
            jl_platform_free(in_req->messageId);
            in_req->messageId = NULL;
        }
    }
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   设备NTP授时 打包NTP请求结构体
 *
 * @param[in] in_req: 请求NTP结构体指针
 * @return 
 *    打包后的json串指针
 * @see None.
 * @note None.
 */
char *joylink_proto_pack_ntp_req(JoyProtoNTPReq *in_req)
{
    if(NULL == in_req) {
        return NULL;
    }

    cJSON *root;
    char *out  = NULL;
    root = cJSON_CreateObject();
    if(NULL == root){
        goto RET;
    }

    // 添加 deviceId
    if (in_req->deviceId != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID, in_req->deviceId);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID, "");
    }

    // 添加 messageId
    if (in_req->messageId != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID, in_req->messageId);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID, "");
    }

    // 添加 设备发送时间
    // FIXME
    char time_string[16];
    jl_platform_memset(time_string, 0, 16 * sizeof(char));
    joylink_ntp_stamp_to_string(&in_req->devSendTime, time_string);
    cJSON_AddStringToObject(root, "deviceSendTime", time_string);

    // 转换成字符串
    out = cJSON_Print(root);

    // 删除 root
    cJSON_Delete(root);

RET:
    return out;
}

/**
 * @brief   设备NTP授时 释放 请求响应信息结构体成员变量
 *
 * @param[in] in_res: NTP请求响应信息结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_ntp_req_res(JoyProtoNTPReqRes *in_res)
{
    if (in_res != NULL) {
        if (in_res->deviceId != NULL) {
            jl_platform_free(in_res->deviceId);
            in_res->deviceId = NULL;
        }
        if (in_res->messageId != NULL) {
            jl_platform_free(in_res->messageId);
            in_res->messageId = NULL;
        }
    }
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   设备NTP授时 解析NTP请求响应消息体
 *
 * @param[in] in_json: 输入的json串
 * @param[in] out_res: NTP请求响应结构体指针
 * @return 
 *    返回值 JoylinkRetCode_E 
 * @see None.
 * @note None.
 */
int32_t joylink_proto_parse_ntp_req_res(char *in_json, JoyProtoNTPReqRes *out_res)
{
    int ret = JOYLINK_RET_FAILED;
    if (NULL == in_json || NULL == out_res) {
        goto RET;
    }
    cJSON *payload = cJSON_Parse(in_json);
    if (NULL == payload) {
        goto RET;
    }
    
    cJSON *pV = NULL;

    // 解析 deviceId
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    if (out_res->deviceId == NULL) {
        out_res->deviceId = (char *) jl_platform_malloc((jl_platform_strlen(pV->valuestring) + 2) * sizeof(char));
    }
    jl_platform_strcpy(out_res->deviceId, pV->valuestring);

    // 解析 messageId
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    if (out_res->messageId == NULL) {
        out_res->messageId = (char *) jl_platform_malloc((jl_platform_strlen(pV->valuestring) + 2) * sizeof(char));
    }
    jl_platform_strcpy(out_res->messageId, pV->valuestring);

    // TODO 解析 code
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_CODE);
    if (NULL != pV) {
        out_res->code = pV->valueint;
    }

    // 解析 设备发送时间
    pV = cJSON_GetObjectItem(payload, "deviceSendTime");
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    joylink_ntp_string_to_stamp(pV->valuestring, &out_res->devSendTime);

    // 解析 服务端接收时间
    pV = cJSON_GetObjectItem(payload, "serverRecvTime");
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    joylink_ntp_string_to_stamp(pV->valuestring, &out_res->serRecvTime);

    // 解析 服务端发送时间
    pV = cJSON_GetObjectItem(payload, "serverSendTime");
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    joylink_ntp_string_to_stamp(pV->valuestring, &out_res->serSendTime);

    // 删除 payload
    cJSON_Delete(payload);

    ret = JOYLINK_RET_SUCCESS;
RET:
    return ret;
}

/**
 * @brief   ntp(set)消息主题 解析ntp设置消息
 *
 * @param[in] in_json: 输入的json串
 * @param[out] out_set: 用于存解析结果的 ntp设置结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note None.
 */
int32_t joylink_proto_parse_ntp_set(char *in_json, JoyProtoNtpSet_t *out_set)
{
    int32_t ret = JOYLINK_RET_FAILED;
    if (NULL == in_json || NULL == out_set) {
        goto RET;
    }
    cJSON *payload = cJSON_Parse(in_json);
    if (NULL == payload) {
        goto RET;
    }

    cJSON *pV = NULL;

    // 解析 deviceId
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    if (out_set->deviceId == NULL) {
        out_set->deviceId = (char *) jl_platform_malloc((jl_platform_strlen(pV->valuestring) + 2) * sizeof(char));
    }
    jl_platform_strcpy(out_set->deviceId, pV->valuestring);

    // 解析 messageId
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    if (out_set->messageId == NULL) {
        out_set->messageId = (char *) jl_platform_malloc((jl_platform_strlen(pV->valuestring) + 2) * sizeof(char));
    }
    jl_platform_strcpy(out_set->messageId, pV->valuestring);

    // 解析 timestamp
    pV = cJSON_GetObjectItem(payload, JOYLINK_PAYLOAD_NODE_NAME_TIMESTAMP);
    if (NULL == pV) {
        cJSON_Delete(payload);
        goto RET;
    }
    out_set->timestamp = pV->valueint;

    // 删除 payload
    cJSON_Delete(payload);

    ret = JOYLINK_RET_SUCCESS;
RET:
    return ret;
}

/**
 * @brief    ntp(set)消息主题 释放ntp设置结构体 成员变量的内存空间
 *
 * @param[in] in_set: ntp设置结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_ntp_set(JoyProtoNtpSet_t *in_set)
{
    if (in_set != NULL) {
        if (in_set->deviceId != NULL) {
            jl_platform_free(in_set->deviceId);
            in_set->deviceId = NULL;
        }
        if (in_set->messageId != NULL) {
            jl_platform_free(in_set->messageId);
            in_set->messageId = NULL;
        }
    }
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   ntp(set/response)消息主题 释放ntp设置响应结构体 成员变量的内存空间
 *
 * @param[in] in_res: ntp设置响应结构体指针
 * @return
 *    返回值 JoylinkRetCode_E
 * @see None.
 * @note 只释放其中成员变量的内存空间
 */
int32_t joylink_proto_free_ntp_set_res(JoyProtoNtpSetRes_t *in_res)
{
    if (in_res != NULL) {
        if (in_res->deviceId != NULL) {
            jl_platform_free(in_res->deviceId);
            in_res->deviceId = NULL;
        }
        if (in_res->messageId != NULL) {
            jl_platform_free(in_res->messageId);
            in_res->messageId = NULL;
        }
        if (in_res->message != NULL) {
            jl_platform_free(in_res->message);
            in_res->message = NULL;
        }
    }
    return JOYLINK_RET_SUCCESS;
}

/**
 * @brief   ntp(set/response)消息主题 打包ntp设置响应
 *
 * @param[in] in_res: 待打包的 ntp设置响应 结构体指针
 * @return
 *    打包后的json串指针
 * @see None.
 * @note None.
 */
char   *joylink_proto_pack_ntp_set_res(JoyProtoNtpSetRes_t *in_res)
{
    if(NULL == in_res) {
        return NULL;
    }

    cJSON *root;
    char *out  = NULL;
    root = cJSON_CreateObject();
    if(NULL == root){
        goto RET;
    }

    // 添加 deviceId
    if (in_res->deviceId != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID, in_res->deviceId);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_DEV_ID, "");
    }

    // 添加 timestamp
    cJSON_AddNumberToObject(root, JOYLINK_PAYLOAD_NODE_NAME_TIMESTAMP, in_res->timestamp);

    // 添加 messageId
    if (in_res->messageId != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID, in_res->messageId);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG_ID, "");
    }

    // 添加 message (可选)
    if (in_res->message != NULL) {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG, in_res->message);
    } else {
        cJSON_AddStringToObject(root, JOYLINK_PAYLOAD_NODE_NAME_MSG, "");
    }

    // 添加 code
    cJSON_AddNumberToObject(root, JOYLINK_PAYLOAD_NODE_NAME_CODE, in_res->code);

    // 转换成字符串
    out = cJSON_Print(root);

    // 删除 root
    cJSON_Delete(root);

RET:
    return out;
}

// end of file
